@@ -7,10 +7,13 @@ installed."""
 import re
 import time
 
-try: import rfc822
+
+try:
+    import rfc822
 except ImportError:
     from email import _parseaddr as rfc822
 
+
 class FeedParserDict(dict):
     keymap = {'channel': 'feed',
               'items': 'entries',
@@ -32,7 +35,7 @@ class FeedParserDict(dict):
             try:
                 return dict.__getitem__(self, 'tags')[0]['term']
             except IndexError:
-                raise KeyError, "object doesn't have key 'category'"
+                raise KeyError("object doesn't have key 'category'")
         elif key == 'enclosures':
             norel = lambda link: FeedParserDict([(name,value) for (name,value) in link.items() if name!='rel'])
             return [norel(link) for link in dict.__getitem__(self, 'links') if link['rel']==u'enclosure']
@@ -84,7 +87,7 @@ class FeedParserDict(dict):
         try:
             return self.__getitem__(key)
         except KeyError:
-            raise AttributeError, "object has no attribute '%s'" % key
+            raise AttributeError("object has no attribute '%s'" % key)
 
     def __hash__(self):
         return id(self)
