@@ -15,10 +15,10 @@ LIMITATIONS:
 """
 
 import re
-import os
-import sys
 import time
-from lxml import etree, html
+import urlparse
+import chardet
+from lxml import etree
 from lxml.html import clean
 
 try:
@@ -35,14 +35,22 @@ xmlns_map = {
     'http://my.netscape.com/rdf/simple/0.9/': 'rss090',
 }
 
-default_cleaner = clean.Cleaner(comments=True, javascript=True,
-        scripts=True, safe_attrs_only=True, page_structure=True,
-        style=True)
+default_cleaner = clean.Cleaner(
+    comments=True,
+    javascript=True,
+    scripts=True,
+    safe_attrs_only=True,
+    page_structure=True,
+    style=True
+)
 
 simple_cleaner = clean.Cleaner(safe_attrs_only=True, page_structure=True)
 
+
 class FakeCleaner(object):
-    def clean_html(self, x): return x
+    def clean_html(self, x):
+        return x
+
 
 class IncompatibleFeedError(Exception):
     pass
@@ -51,25 +59,35 @@ fake_cleaner = FakeCleaner()
 
 # --- text utilities ---
 
+
 def unicoder(txt, hint=None, strip=True):
     if txt is None:
         return None
     if strip:
         txt = txt.strip()
     if hint:
-        try: return txt.decode(hint)
-        except: return unicoder(txt)
+        try:
+            return txt.decode(hint)
+        except:
+            return unicoder(txt)
     try:
         return txt.decode('utf-8')
     except:
-        try: return txt.decode('latin-1')
-        except: return txt
+        try:
+            return txt.decode('latin-1')
+        except:
+            try:
+                return txt.decode('iso-8859')
+            except:
+                return txt
+
 
 def first_text(xpath_result, default='', encoding='utf-8'):
     if xpath_result:
         return unicoder(xpath_result[0].text, encoding) or default
     return default
 
+
 def strip_outer_tag(text):
     """Strips the outer tag, if text starts with a tag.  Not entity aware;
     designed to quickly strip outer tags from lxml cleaner output.  Only
@@ -82,24 +100,27 @@ def strip_outer_tag(text):
         return stripped[stripped.index('>')+1:stripped.rindex('<')]
     return text
 
-nsre = re.compile(r'xmlns=[\'"](.+?)[\'"]')
+nsre = re.compile(r'xmlns\s*=\s*[\'"](.+?)[\'"]')
+
+
 def strip_namespace(document):
     if document[:1000].count('xmlns') > 5:
-        if 'xmlns=' not in document[:1000]:
+        if 'xmlns' not in document[:1000]:
             return None, document
-    elif 'xmlns=' not in document[:400]:
+    elif 'xmlns' not in document[:400]:
         return None, document
     match = nsre.search(document)
     if match:
         return match.groups()[0], nsre.sub('', document)
     return None, document
 
+
 def munge_author(author):
     """If an author contains an email and a name in it, make sure it is in
     the format: "name (email)"."""
     # this loveliness is from feedparser but was not usable as a function
     if '@' in author:
-        emailmatch = re.search(ur'''(([a-zA-Z0-9\_\-\.\+]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?))(\?subject=\S+)?''', author)
+        emailmatch = re.search(r"(([a-zA-Z0-9\_\-\.\+]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([a-zA-Z0-9\-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?))(\?subject=\S+)?", author, re.UNICODE)
         if emailmatch:
             email = emailmatch.group(0)
             # probably a better way to do the following, but it passes all the tests
@@ -118,35 +139,35 @@ def munge_author(author):
 
 # --- common xml utilities ---
 
+
 def reverse_namespace_map(nsmap):
     d = fpnamespaces.copy()
-    d.update(dict([(v,k) for (k,v) in nsmap.iteritems()]))
+    d.update(dict([(v, k) for (k, v) in nsmap.iteritems()]))
     return d
 
+
 def base_url(root):
     """Determine the base url for a root element."""
-    for attr,value in root.attrib.iteritems():
+    for attr, value in root.attrib.iteritems():
         if attr.endswith('base') and 'http' in value:
             return value
     return None
 
+
 def full_href(href, base=None):
-    if base is None:
-        return href
-    if href.startswith('javascript:'):
-        return href
-    if 'http://' in href or 'https://' in href:
-        return href
-    return '%s/%s' % (base.rstrip('/'), href.lstrip('/'))
+    return urlparse.urljoin(base, href)
+
 
 def full_href_attribs(attribs, base=None):
-    if base is None: return dict(attribs)
+    if base is None:
+        return dict(attribs)
     d = dict(attribs)
-    for key,value in d.iteritems():
+    for key, value in d.iteritems():
         if key == 'href':
             d[key] = full_href(value, base)
     return d
 
+
 def clean_ns(tag):
     """Return a tag and its namespace separately."""
     if '}' in tag:
@@ -154,28 +175,33 @@ def clean_ns(tag):
         return split[0].strip('{'), split[-1]
     return '', tag
 
+
 def xpath(node, query, namespaces={}):
     """A safe xpath that only uses namespaces if available."""
     if namespaces and 'None' not in namespaces:
         return node.xpath(query, namespaces=namespaces)
     return node.xpath(query)
 
+
 def innertext(node):
     """Return the inner text of a node.  If a node has no sub elements, this
     is just node.text.  Otherwise, it's node.text + sub-element-text +
     node.tail."""
-    if not len(node): return node.text
+    if not len(node):
+        return node.text
+
     return (node.text or '') + ''.join([etree.tostring(c) for c in node]) + (node.tail or '')
 
+
 class SpeedParserEntriesRss20(object):
     entry_xpath = '/rss/item | /rss/channel/item'
     tag_map = {
-        'pubDate' : 'date',
-        'pubdate' : 'date',
+        'pubDate': 'date',
+        'pubdate': 'date',
         'date': 'date',
-        'updated' : 'date',
-        'modified' : 'date',
-        'link' : 'links',
+        'updated': 'date',
+        'modified': 'date',
+        'link': 'links',
         'title': 'title',
         'creator': 'author',
         'author': 'author',
@@ -192,7 +218,11 @@ class SpeedParserEntriesRss20(object):
         'media_thumbnail': 'media_thumbnail',
         'media:group': 'media_group',
         'itunes:summary': 'content',
+        'itunes:keywords': 'content',
+        'itunes:image': 'media_thumbnail',
+        'itunes:author': 'author',
         'gr:annotation': 'annotation',
+        'enclosure': 'enclosures',
     }
 
     def __init__(self, root, namespaces={}, version='rss20', encoding='utf-8', feed={},
@@ -210,7 +240,8 @@ class SpeedParserEntriesRss20(object):
         entries = []
         for obj in self.entry_objects:
             d = self.parse_entry(obj)
-            if d: entries.append(d)
+            if d:
+                entries.append(d)
         self.entries = entries
 
     def clean(self, text):
@@ -248,7 +279,7 @@ class SpeedParserEntriesRss20(object):
         if not lacks_summary and lacks_content:
             e['content'] = [{'value': e.summary}]
 
-        # feedparser sometimes copies the first content value into the 
+        # feedparser sometimes copies the first content value into the
         # summary field when summary was completely missing;  we want
         # to do that as well, but avoid the case where summary was given as ''
         if lacks_summary and not lacks_content:
@@ -274,8 +305,9 @@ class SpeedParserEntriesRss20(object):
     def parse_title(self, node, entry, ns=''):
         if ns in ('media',) and 'title' in entry:
             return
-        title = unicoder(node.text) or ''
-        title = strip_outer_tag(self.clean(title))
+        title = unicoder(node.text)
+        if title is not None:
+            title = strip_outer_tag(self.clean(title.strip()))
         entry['title'] = title or ''
 
     def parse_author(self, node, entry, ns=''):
@@ -286,8 +318,10 @@ class SpeedParserEntriesRss20(object):
             return
         name, email = None, None
         for child in node:
-            if child.tag == 'name': name = unicoder(child.text or '')
-            if child.tag == 'email': email = unicoder(child.text or '')
+            if child.tag == 'name':
+                name = unicoder(child.text or '')
+            if child.tag == 'email':
+                email = unicoder(child.text or '')
         if name and not email:
             entry['author'] = munge_author(name)
         elif not name and not email:
@@ -296,7 +330,8 @@ class SpeedParserEntriesRss20(object):
             entry['author'] = '%s (%s)' % (name, email)
 
     def parse_guid(self, node, entry, ns=''):
-        entry['guid'] = unicoder(node.text)
+        if node.text:
+            entry['guid'] = unicoder(node.text)
 
     def parse_annotation(self, node, entry, ns='gr'):
         if entry.get('author', '') and 'unknown' not in entry['author'].lower():
@@ -306,13 +341,29 @@ class SpeedParserEntriesRss20(object):
                 self.parse_author(child, entry, ns='')
 
     def parse_links(self, node, entry, ns=''):
+
         if unicoder(node.text):
-            entry['link'] = full_href(unicoder(node.text).strip('#'), self.baseurl)
+            entry['link'] = full_href(
+                unicoder(node.text).strip('#'),
+                self.baseurl
+            )
+
         if 'link' not in entry and node.attrib.get('rel', '') == 'alternate' and 'href' in node.attrib:
-            entry['link'] = full_href(unicoder(node.attrib['href']).strip('#'), self.baseurl)
+            entry['link'] = full_href(
+                unicoder(node.attrib['href']).strip('#'),
+                self.baseurl
+            )
         if 'link' not in entry and 'rel' not in node.attrib and 'href' in node.attrib:
-            entry['link'] = full_href(unicoder(node.attrib['href']).strip('#'), self.baseurl)
-        entry.setdefault('links', []).append(full_href_attribs(node.attrib, self.baseurl))
+            entry['link'] = full_href(
+                unicoder(node.attrib['href']).strip('#'),
+                self.baseurl
+            )
+
+        if node.attrib:
+            entry.setdefault('links', []).append(
+                full_href_attribs(node.attrib, self.baseurl)
+            )
+
         # media can be embedded within links..
         for child in node:
             ns, tag = clean_ns(child.tag)
@@ -320,7 +371,8 @@ class SpeedParserEntriesRss20(object):
                 self.parse_media_content(child, entry)
 
     def parse_comments(self, node, entry, ns=''):
-        if 'comments' in entry and ns: return
+        if 'comments' in entry and ns:
+            return
         entry['comments'] = strip_outer_tag(self.clean(unicoder(node.text)))
 
     def parse_content(self, node, entry, ns=''):
@@ -332,7 +384,8 @@ class SpeedParserEntriesRss20(object):
         entry.setdefault('content', []).append({'value': content or ''})
 
     def parse_summary(self, node, entry, ns=''):
-        if ns in ('media', ): return
+        if ns in ('media', ):
+            return
         if ns == 'itunes' and entry.get('summary', None):
             return
         if 'content' in entry:
@@ -358,12 +411,19 @@ class SpeedParserEntriesRss20(object):
             elif child.tag.endswith('thumbnail'):
                 self.parse_media_thumbnail(child, entry)
 
+    def parse_enclosures(self, node, entry, ns=''):
+        entry.setdefault('links', []).append(
+            dict(node.attrib, rel='enclosure')
+        )
+
     def entry_list(self):
         return self.entries
 
+
 class SpeedParserEntriesRdf(SpeedParserEntriesRss20):
     entry_xpath = '/rdf:RDF/item | /rdf:RDF/channel/item'
 
+
 class SpeedParserEntriesAtom(SpeedParserEntriesRss20):
     entry_xpath = '/feed/entry'
 
@@ -373,23 +433,26 @@ class SpeedParserEntriesAtom(SpeedParserEntriesRss20):
         else:
             super(SpeedParserEntriesAtom, self).parse_summary(node, entry, ns)
 
+
 class SpeedParserFeedRss20(object):
     channel_xpath = '/rss/channel'
     tag_map = {
-        'title' : 'title',
-        'description' : 'subtitle',
-        'tagline' : 'subtitle',
-        'subtitle' : 'subtitle',
-        'link' : 'links',
+        'title': 'title',
+        'description': 'subtitle',
+        'tagline': 'subtitle',
+        'subtitle': 'subtitle',
+        'link': 'links',
         'pubDate': 'date',
-        'updated' : 'date',
-        'modified' : 'date',
+        'updated': 'date',
+        'modified': 'date',
         'date': 'date',
-        'generator' : 'generator',
+        'generator': 'generator',
         'generatorAgent': 'generator',
-        'language' : 'lang',
+        'language': 'lang',
         'id': 'id',
-        'lastBuildDate' : 'date',
+        'lastBuildDate': 'date',
+        'itunes:summary': 'subtitle',
+        'itunes:image': 'image'
     }
 
     def __init__(self, root, namespaces={}, encoding='utf-8', type='rss20', cleaner=default_cleaner,
@@ -424,7 +487,6 @@ class SpeedParserFeedRss20(object):
             if mapping:
                 getattr(self, 'parse_%s' % mapping)(child, feed, nslookup[ns])
 
-
         # this copies feedparser behavior if, say, xml:lang is defined in the
         # root feed element, even though this element tends to have garbage like
         # "utf-8" in it rather than an actual language
@@ -443,7 +505,6 @@ class SpeedParserFeedRss20(object):
             if not outer_tag:
                 txt = self.cleaner.clean_html(text)
                 frag = lxml.html.fragment_fromstring(txt)
-                import ipdb; ipdb.set_trace();
             return self.cleaner.clean_html(text)
         return text
 
@@ -453,6 +514,9 @@ class SpeedParserFeedRss20(object):
     def parse_subtitle(self, node, feed, ns=''):
         feed['subtitle'] = strip_outer_tag(self.clean(unicoder(node.text))) or ''
 
+    def parse_image(self, node, entry, ns='media'):
+        entry['image'] = dict(node.attrib)
+
     def parse_links(self, node, feed, ns=''):
         if node.text:
             feed['link'] = full_href(unicoder(node.text).strip('#'), self.baseurl)
@@ -488,14 +552,22 @@ class SpeedParserFeedRss20(object):
     def feed_dict(self):
         return self.feed
 
+
 class SpeedParserFeedAtom(SpeedParserFeedRss20):
     channel_xpath = '/feed'
 
+
 class SpeedParserFeedRdf(SpeedParserFeedRss20):
     channel_xpath = '/rdf:RDF/channel'
 
+
 class SpeedParser(object):
-    def __init__(self, content, cleaner=default_cleaner, unix_timestamp=False):
+
+    version_map = {
+        'rss2': 'rss20',
+    }
+
+    def __init__(self, content, cleaner=default_cleaner, unix_timestamp=False, encoding=None):
         self.cleaner = cleaner
         self.xmlns, content = strip_namespace(content)
         self.unix_timestamp = unix_timestamp
@@ -509,8 +581,10 @@ class SpeedParser(object):
         else:
             self.tree = tree.getroottree()
             self.root = tree
-        self.encoding = self.parse_encoding()
+        self.encoding = encoding if encoding else self.parse_encoding()
         self.version = self.parse_version()
+        if self.version in self.version_map:
+            self.version = self.version_map[self.version]
         if 'unk' in self.version:
             raise IncompatibleFeedError("Could not determine version of this feed.")
         self.namespaces = self.parse_namespaces()
@@ -523,7 +597,7 @@ class SpeedParser(object):
         root_tag = root_tag.lower()
         vers = 'unk'
         if self.xmlns and self.xmlns.lower() in xmlns_map:
-            value =  xmlns_map[self.xmlns.lower()]
+            value = xmlns_map[self.xmlns.lower()]
             if value == 'rss10' and root_tag == 'rss':
                 value = 'rss010'
             if not (value.startswith('atom') and root_tag == 'rss'):
@@ -561,7 +635,7 @@ class SpeedParser(object):
         )
         if version in ('rss20', 'rss092', 'rss091', 'rss'):
             return SpeedParserFeedRss20(self.root, **kwargs).feed_dict()
-        if version == 'rss10':
+        if version in ('rss090', 'rss10'):
             return SpeedParserFeedRdf(self.root, **kwargs).feed_dict()
         if version in ('atom10', 'atom03'):
             return SpeedParserFeedAtom(self.root, **kwargs).feed_dict()
@@ -572,7 +646,7 @@ class SpeedParser(object):
             cleaner=self.cleaner, feed=self.feed, unix_timestamp=self.unix_timestamp)
         if version in ('rss20', 'rss092', 'rss091', 'rss'):
             return SpeedParserEntriesRss20(self.root, **kwargs).entry_list()
-        if version == 'rss10':
+        if version in ('rss090', 'rss10'):
             return SpeedParserEntriesRdf(self.root, **kwargs).entry_list()
         if version in ('atom10', 'atom03'):
             return SpeedParserEntriesAtom(self.root, **kwargs).entry_list()
@@ -591,13 +665,14 @@ class SpeedParser(object):
             result['encoding'] = self.encoding
 
 
-def parse(document, clean_html=True, unix_timestamp=False):
+def parse(document, clean_html=True, unix_timestamp=False, encoding=None):
     """Parse a document and return a feedparser dictionary with attr key access.
     If clean_html is False, the html in the feed will not be cleaned.  If
     clean_html is True, a sane version of lxml.html.clean.Cleaner will be used.
     If it is a Cleaner object, that cleaner will be used.  If unix_timestamp is
-    True, the date information will be a numerical unix
-    timestamp rather than a struct_time."""
+    True, the date information will be a numerical unix timestamp rather than a
+    struct_time.  If encoding is provided, the encoding of the document will be
+    manually set to that."""
     if isinstance(clean_html, bool):
         cleaner = default_cleaner if clean_html else fake_cleaner
     else:
@@ -607,13 +682,28 @@ def parse(document, clean_html=True, unix_timestamp=False):
     result['entries'] = []
     result['bozo'] = 0
     try:
-        parser = SpeedParser(document, cleaner, unix_timestamp)
+        parser = SpeedParser(document, cleaner, unix_timestamp, encoding)
         parser.update(result)
-    except Exception, e:
+    except Exception as e:
+        if isinstance(e, UnicodeDecodeError) and encoding is True:
+            encoding = chardet.detect(document)['encoding']
+            document = document.decode(encoding, 'replace').encode('utf-8')
+            return parse(document, clean_html, unix_timestamp, encoding)
         import traceback
         result['bozo'] = 1
         result['bozo_exception'] = e
         result['bozo_tb'] = traceback.format_exc()
     return result
 
+if __name__ == '__main__':
+    import sys
 
+    if len(sys.argv) != 2:
+        print("Must provide filename of feed.")
+    filename = sys.argv[1]
+    feed = open(filename).read()
+    if '-- END TRACEBACK --' in feed:
+        feed = feed.split('-- END TRACEBACK --')[1].strip()
+
+    import pprint
+    pprint.pprint(parse(feed))
